@@ -8,6 +8,8 @@ our $VERSION = '0.85';
 
 __PACKAGE__->allow_apparent_reload(1);
 
+our %Made_Method_Custom;
+
 sub import
 {
   my($class) = shift;
@@ -117,7 +119,7 @@ sub __make_methods
     METHOD: while(my($name, $code) = each(%$make))
     {
       Carp::croak "${class}::method_type(...) - key for $name is not a code ref!"
-        unless(ref $code eq 'CODE');
+        unless(ref $code eq 'CODE' || (ref $code eq 'HASH' && $code->{'make_method'}));
 
       if(my $code = $target_class->can($name))
       {
@@ -138,7 +140,18 @@ sub __make_methods
       }
 
       no warnings;
-      *{"${target_class}::$name"} = $code;
+
+      if(ref $code eq 'CODE')
+      {
+        *{"${target_class}::$name"} = $code;
+      }
+      else
+      {
+        # XXX: Must track these separately because they do not show up as
+        # XXX: being named __ANON__ when fetching the sub_identity()
+        $Made_Method_Custom{$target_class}{$name}++;
+        $code->{'make_method'}($name, $target_class, $options);
+      }
     }
   }
 
@@ -150,13 +163,14 @@ sub apparently_made_method
   my($class, $code) = @_;
   my($mm_class, $name) = $class->sub_identity($code);
   return 0  unless($class && $name);
-  return ($mm_class eq $class && $name eq '__ANON__') ? 1 : 0;
+  return (($mm_class eq $class && $name eq '__ANON__') ||
+          $Made_Method_Custom{$mm_class}{$name}) ? 1 : 0;
 }
 
 # Code from Sub::Identify
 sub sub_identity
 {
-  my($clas, $code) = @_;
+  my($class, $code) = @_;
 
   my @id;
 
